// thornberry/employee/employeeEditor.js
define([
	"dojo/_base/declare",
	"thornberry/object/ObjectEditor",
	"thornberry/object/DatabaseObjectModel",
	"thornberry/object/PropertyModel",
	"thornberry/form/searchBox",
	"thornberry/object/TableManager",
	"thornberry/serverRequest",
	"thornberry/form/DragNDropSelect",
	"dojo/text!./templates/employeeEditor.html",
	"dojo/date/locale",
	"dojo/dom-style",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom-geometry",
	"dojo/store/Memory",
	"dojo/store/Observable",
	"dojo/data/ObjectStore",
	"dojo/query",
	"dojo/Deferred",
	"dojo/aspect",
	"dojo/topic",
	"dijit/form/FilteringSelect",
	"dijit/form/Select",
	"dijit/form/DateTextBox",
	"thornberry/form/TimeTextBox",
	"dijit/form/ValidationTextBox",
	"dijit/form/ComboBox",
	"dijit/form/Textarea",
	"dijit/form/CheckBox",
	"dijit/form/Form",
	"dijit/form/DropDownButton",
	"dijit/form/TextBox",
	"dijit/TooltipDialog",
	"dijit/Tooltip",
	"dijit/form/Button",
	"dijit/popup",
	"dijit/registry",
	"dojox/form/CheckedMultiSelect",
	"dojo/on",
	"dojo/date",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/data/ItemFileReadStore",
	"gridx/Grid",
	"gridx/core/model/cache/Sync",
	"gridx/modules/select/Row",
	"gridx/modules/extendedSelect/Row",
	"gridx/modules/IndirectSelectColumn",
	"gridx/modules/Filter",
	"gridx/modules/SingleSort",
	"gridx/core/model/extensions/FormatSort",
	"gridx/modules/VirtualVScroller",
	"dojo/NodeList-dom",
	"dijit/layout/StackContainer"
], function(declare,ObjectEditor,DatabaseObjectModel,PropertyModel,SearchBox,TableManager,serverRequest,DragNDropSelect,template,locale,domStyle,domClass,domConstruct,domGeom,Memory,Observable,ObjectStore,query,Deferred,aspect,topic,FilteringSelect,Select,DateTextBox,TimeTextBox,ValidationTextBox,ComboBox,Textarea,CheckBox,Form,DropDownButton,TextBox,TooltipDialog,Tooltip,Button,popup,registry,CheckedMultiSelect,on,date,lang,arrayUtils,ItemStore,Grid,Cache,SelectRow,ExtendedSelectRow,IndirectSelectColumn,Filter,SingleSort,FormatSort,VirtualVScroller){
		/*
			Author: rwitherell
			Created: 2017
			History:
			2023-03-23 DJA ND-12262 Added custom User Account Status text handler (accountStatus)
									Added Unlock boolean field (unlockAccount)
									./templates/employeeEditor.html - Added User Account Status text and Unlock checkbox
			2019-07-09 WRA ND-9062 Added addressCoordinates field
			2018-06-13 CGS ND-6998 Changed the Preference property to read "Patient Characteristics"
			2018-01-23 WRA ND-4175 Added dndPasswordResetFlag property.
		*/
		var EmployeeEditor = declare([ObjectEditor], {
			templateString: template,
			baseClass: "EmployeeEditor",
			
			availabilityModel: null,
			availabilityEditor: null,
			
			contactStore: null,
			
			postMixInProperties: function() {
				var t = this;
				t.inherited(arguments);
				
				t.contactStore = new Observable(new Memory());
				
				t.objectModel.addPropertyDefinition({
					name: "status",
					displayName: "Status",
					type: "enum",
					displayIndex: {
						1: "Active",
						2: "Inactive"
					},
					formatter: function(value,object) {
						return (object.get("inactiveDate") === null ? this.displayIndex[1] : this.displayIndex[2]);
					},
					getInternal: function(value,object) {
						var active = (typeof value === "object" ? value.join(",") : value);
						object.set("inactiveUser",(active == 1 ? "" : userId));
						object.set("inactiveDate",(active == 1 ? null : new Date()));
						return "";
					},
					getExternal: function(value,object) {
						var active = (object.get("inactiveDate") === null ? true : false);
						return (active ? [1] : [2]);
					}
				});
				
				t.objectModel.addPropertyDefinition({
					name: "unlockAccount",
					displayName: "Unlock",
					type: "boolean"
				});
				
				t.getPropertyModel("name").getInternal = function(value) {return (this._inLoad ? value : value.toString().replace(/(~|\^|`|;|'|")/g,""));};
				
				t.getPropertyModel("id").fieldProperties["style"] = "width:120px";
				t.getPropertyModel("id").fieldProperties["uppercase"] = true;
				t.getPropertyModel("id").fieldProperties["regExp"] = "\\S*"; 						// No whitespace
				t.getPropertyModel("middleInitial").fieldProperties["style"] = "width:40px";
				t.getPropertyModel("middleInitial").fieldProperties["uppercase"] = true;
				t.getPropertyModel("initials").fieldProperties["style"] = "width:60px";
				t.getPropertyModel("initials").fieldProperties["uppercase"] = true;
				t.getPropertyModel("ssn").fieldProperties["regExp"] = "\\d{9}";
				t.getPropertyModel("ssn").fieldProperties["invalidMessage"] = "Enter the nine-digit SSN with no dashes or spaces";
				t.getPropertyModel("state").fieldProperties["style"] = "width:60px";
				t.getPropertyModel("zip").fieldProperties["regExp"] = "\\d{5}(-\\d{4})?";
				t.getPropertyModel("zip").fieldProperties["style"] = "width:90px";
				t.getPropertyModel("teams").fieldProperties["size"] = 8;
				t.getPropertyModel("employeeType").fieldProperties["class"] = "multiSelectLabelSmall";
				t.getPropertyModel("status").fieldProperties["class"] = "multiSelectLabelSmall";
				t.getPropertyModel("contract").fieldProperties["class"] = "multiSelectLabelSmallest";
				t.getPropertyModel("fluVaccinationStatus").fieldProperties["class"] = "multiSelectLabelGrand";
				t.getPropertyModel("fluVaccinationStatus").noDefault = true;
				t.getPropertyModel("fluStatusDate").fieldProperties["constraints"] = {max:new Date()};
				t.getPropertyModel("skills").fieldProperties["size"] = 8;
				t.getPropertyModel("preferences").fieldProperties["size"] = 8;
				t.getPropertyModel("zones").fieldProperties["size"] = 8;
				t.getPropertyModel("userId").fieldProperties["style"] = "width:120px";
				t.getPropertyModel("userId").fieldProperties["regExp"] = "\\S*"; 					// No whitespace
				t.getPropertyModel("accessType").fieldProperties["class"] = "multiSelectLabelMedium";
				t.getPropertyModel("types").fieldProperties["size"] = 8;
				t.getPropertyModel("primaryPhone").fieldProperties["regExp"] = "(\\d{3}-\\d{3}-\\d{4}|\\d{10})";
				t.getPropertyModel("primaryPhone").fieldProperties["invalidMessage"] = "The phone number must be in ###-###-#### or ########## format";
				t.getPropertyModel("otherPhone").fieldProperties["regExp"] = "(\\d{3}-\\d{3}-\\d{4}|\\d{10})";
				t.getPropertyModel("otherPhone").fieldProperties["invalidMessage"] = "The phone number must be in ###-###-#### or ########## format";
				t.getPropertyModel("cellPhone").fieldProperties["regExp"] = "(\\d{3}-\\d{3}-\\d{4}|\\d{10})";
				t.getPropertyModel("cellPhone").fieldProperties["invalidMessage"] = "The phone number must be in ###-###-#### or ########## format";
				
				var teamsPropertyModel = t.getPropertyModel("teams");
				lang.mixin(teamsPropertyModel,{getField: function(node) {return new DragNDropSelect(this.getFieldProperties(),node);}});
				
				var typesPropertyModel = t.getPropertyModel("types");
				lang.mixin(typesPropertyModel,{getField: function(node) {return new DragNDropSelect(this.getFieldProperties(),node);}});
				
				if (hasOn) {
					// HAS specific restrictions
					t.getPropertyModel("name").maxLength = 70;
					t.getPropertyModel("firstName").maxLength = 20;
					t.getPropertyModel("lastName").maxLength = 25;
					t.getPropertyModel("fedTaxId").maxLength = 12;
					t.getPropertyModel("street1").maxLength = 30;
					t.getPropertyModel("street2").maxLength = 30;
				}
				
				if (dialNDocOn) {
					t.getPropertyModel("dndAccessCode").fieldProperties["regExp"] = "\\d{4,7}";
					t.getPropertyModel("dndAccessCode").fieldProperties["invalidMessage"] = "The access code must be between 4 and 7 numbers";
					t.getPropertyModel("dndAccessCode").fieldProperties["style"] = "width:100px";
				}

				if (geocodeFieldsOn) {
					t.getPropertyModel("addressCoordinates").fieldProperties["regExp"] = "[-+]?([1-8]?\\d(\\.\\d+)?|90(\\.0+)?),\\s?[-+]?(180(\\.0+)?|((1[0-7]\\d)|([1-9]?\\d))(\\.\\d+)?)"
					t.getPropertyModel("addressCoordinates").fieldProperties["invalidMessage"] = "Manual address coordinate override (latitude, longitude). If unsure, leave blank"
				}
				
				t.getPropertyModel("state").getField = function(srcNodeRef) {
					var properties = this.getFieldProperties();
					properties.store = this.getOptionStore();
					return new FilteringSelect(properties,srcNodeRef);
				};
				
				var checkedMultiSelectProperties = ["volunteerLanguages","volunteerTypes","volunteerSkills","volunteerRestrictions","volunteerLocations"];
				arrayUtils.forEach(checkedMultiSelectProperties,function(property) {
					var propertyModel = t.getPropertyModel(property);
					propertyModel.fieldProperties.style = "margin-bottom:12px;";
					propertyModel.fieldProperties.multiple = true;
					propertyModel.fieldProperties["class"] = "multiSelectLabelLarge";
					lang.mixin(propertyModel,{getField: function(node) {return new CheckedMultiSelect(this.getFieldProperties(),node);}});
				});
				
				var fluStatusModel = t.getPropertyModel("fluVaccinationStatus");
				fluStatusModel.getField = function(node) {
					//var t = this;
					//var properties = this.getFieldProperties();
					//properties.multiple = false;
					return lang.hitch(fluStatusModel,t.getPropertyModel("employeeType").getField)(node);
					//return new CheckedMultiSelect(properties,node);
				};
				fluStatusModel.getOptionStore = function() {
					return new Memory({data:arrayUtils.map(this.table.data,function(item) {return {id:item.id,name:item.description};})});
				};
				fluStatusModel.getInternal = function(value) {
					return (typeof value === "object" ? value.join(",") : value);
				};
				fluStatusModel.getExternal = function(value) {
					return (value ? value.toString().split(",") : []);
				};
				
				t.getPropertyModel("notes").getField = function(node) {
					return new Textarea(this.getFieldProperties(),node);
				};
			},
			
			buildRendering: function() {
				var t = this;
				t.inherited(arguments);
				
				t.passwordField = new ValidationTextBox({
					value: "",
					type: "password",
					style: "width:135px",
					maxLength: 64
				},t.passwordNode);
				t.passwordField.startup();
				t.fields.push(t.passwordField);
				t.own(on(t.passwordField,"change",function(value) {
					if (value !== "") {
						if (value !== "********") {
							topic.publish("preventSave",true);
							serverRequest("NDoc.User.Employee","CheckPassword",{userId:t.object.get("userId"),password:value}).then(function(result) {
								if (!result.valid) {
									actionShowMessage(result.warnings+result.passwordRequirementsHTML);
									t.resetPasswordField();
									t.updateFieldPinking();
								} else {
									t.set("modified",true);
								}
								topic.publish("preventSave",false);
							},function() {
								actionShowMessage("There was an error verifying the new password. Please rechart the last value you entered.");
								topic.publish("preventSave",false);
								t.resetPasswordField();
								t.updateFieldPinking();
							});
						} else {
							t.resetPasswordField();
						}
					}
					t.updateFieldPinking();
				}));
				
				if (!payrollOn) {
					domStyle.set(t.payrollDiv,"display","none");
				}
				
				if (!dialNDocOn) {
					domStyle.set(t.dialNDocDiv,"display","none");
				}
				
				if (!billingComponentConfigurationOn) {
					domStyle.set(t.billingAccessDiv,"display","none");
				}

				if (!geocodeFieldsOn) {
					domStyle.set(t.geocodeFieldsDiv,"display","none");
				}
				
				t.own(aspect.after(t,"_setObjectAttr",function() {
					if (isHospice) {
						for (var i = t.contactStore.data.length - 1; i >= 0; i--) {
							t.contactStore.remove(t.contactStore.data[i].id);
						}
						arrayUtils.forEach(t.object.contacts,function(contact) {
							t.contactStore.put(contact);
						});
					}
					
					if (t.object.id == employeeId && arrayUtils.indexOf(t.object.get("types"),996) !== -1) {
						t.disable("types");
					}
					
					t.require("password",(t.userId !== ""));
					if (t.object.get("userId") !== "") {
						t.resetPasswordField();
					} else {
						t.passwordField.set("value","");
					}
					
					/*if (t.object && !t.object.fluVaccinationStatus) {
						t.object.set("fluVaccinationStatus",[4]);
					}*/
					
					// default to option 2
					if (t.object && !t.object.accessType) {
						t.object.set("accessType",2);
					}

					t.notesField.defer("resize");
					
					var availabilityArray = [];
					arrayUtils.forEach(t.object.availability,function(availability) {
						availabilityArray.push(new t.Availability(availability));
					});
					
					t.availabilityGrid.select.row.clear();
					t.availabilityGrid.setStore(new Memory({data:availabilityArray}));
					t.availabilityEditor.set("object",null);
					t.availabilityCalendar.set("value",new Date());
				},true));
				
				if (isHospice) {
					domStyle.set(t.contactLinkNode,"display","block");
					t.fields.push(t.contactSearch);
					t.contactSearch.getSearchResults = function(searchText) {
						var deferred = new Deferred();
						serverRequest("Contact.Utility","Search",{
							searchText:searchText,
							unlinkedOnly:true
						}).then(function(results) {
							deferred.resolve(new Memory({data:results}));
						},function() {
							actionShowMessage("There was an error performing the search. Please try again.");
						});
						return deferred.promise;
					};
					t.own(on(t.contactSearch,"change",function(event) {
						var item = t.contactSearch.store.get(t.contactSearch.get("value"));
						if (item) {
							t.contactStore.put(item);
							t.contactSearch.set("value","");
							t.set("modified",true);
						}
					}));
					t.contactSearch.set("disabled",true);
				}
				
				t.createAvailabilityEditor();
			},
			
			postCreate: function() {
				var t = this;
				t.inherited(arguments);
				
				t.own(aspect.after(t,"_setActionTypeAttr",function(value) {
					t.title = value + " Employee";
					
					t.contactSearch.set("value","");
					
					domStyle.set(t.availabilityActionPane.domNode,"display",("actionType",(value !== "View" ? "" : "none")));
					t.availabilityEditorContainer.resize();
					
					if (value === "View" && t.availabilityStack.selectedChildWidget === t.availabilityEditorPane) {
						t.availabilityCancelButton.onClick();
					}
					
					if (value !== "View") {
						// Force the inactiveDate change logic to run to disable fields
						t.object.set("inactiveDate",t.object.get("inactiveDate"));
					}
					t.disable("status",(value === "Add" ? true : false));

					/*t.disable("gpsLookupButton", (value === "View"));
					t.disable("showMapButton", (value === "View"));
					t.disable("getGpsButton", (value === "View"));*/
				},true));
				
				t.own(on(t.availabilityPane,"hide",function() {
					if (t.availabilityStack.selectedChildWidget === t.availabilityEditorPane) {
						t.availabilityCancelButton.onClick();
					}
				}));
				
				t.own(aspect.after(t,"okToSave",function(value) {
					var t = this;
					if (!value) {
						var focusNode = null;
						var invalidNodes = query(".dijitError",t.containerNode);
						if (invalidNodes.length > 0) {
							focusNode = invalidNodes[0];
						} else {
							var requiredNodes = query(".pinkBackground",t.containerNode);
							if (requiredNodes.length > 0) {
								focusNode = requiredNodes[0];
							}
						}
						
						if (focusNode) {
							// Select tab and focus widget
							var widget = registry.getEnclosingWidget(focusNode);
							
							var parent = widget.getParent();
							while (parent) {
								if (parent.declaredClass === "dijit.layout.ContentPane" && parent.getParent() === t.tabContainer) {
									t.tabContainer.selectChild(parent);
									widget.focus();
									parent = null;
								} else {
									parent = parent.getParent();
								}
							}
						}
					}
					return value;
				}));
				
				t.registerChangeLogic("userId",function(value) {
					var t = this;
					if (!t.inObjectLoad) {
						if (value !== t.objectCopy.get("userId")) {
							if (value === "") {
								actionShowMessage("This User ID cannot be removed. Please inactivate");
								t.object.set("userId",t.objectCopy.get("userId"));
								t.resetPasswordField();
							} else if (value.toString()[0] === "*" || value.toString().toUpperCase() === "TTLCJ") {
								actionShowMessage(value+ " is an invalid User ID");
								t.object.set("userId",t.objectCopy.get("userId"));
								t.resetPasswordField();
							} else {
								if (t.objectCopy.get("userId") !== "") {
									actionShowMessage("Once a user ID is linked to an employee, it cannot be removed. To change this employee's user ID "+
										"<ol>"+
											"<li>Change the employee ID. This carries the user ID along with the new employee ID and inactivates the old employee ID</li>"+
											"<li>Look up the old employee ID. Reactivate this account and enter the new user ID</li>"+
											"<li>Move any scheduled visits and patient assignments to the new user ID</li>"+
											"<li>Inactivate the new employee ID created in step 1</li>"+
										"</ol>");
									t.object.set("userId",t.objectCopy.get("userId"));
								} else {
									topic.publish("preventSave",true);
									DatabaseObjectModel.query("NDoc.User.UserMapping",{idUpper:value.toString().toUpperCase()},["primaryTeam"]).then(function(results) {
										if (results.length > 0) {
											actionShowMessage("Invalid: "+value+" is already assigned");
											t.userIdField.set("value",t.objectCopy.get("userId"));
											t.resetPasswordField();
										} else {
											if (t.objectCopy.get("userId") !== "") {
												actionShowMessage("Warning: you are changing this employee's user ID. Existing charting, medications, and orders will remain associated with the old User ID. The old user ID will be inactivated, and will no longer be able to log in to NDoc. Do you wish to proceed?",[
													{label:"OK"},
													{label:"Cancel",callback:function() {
														t.object.set("userId",t.objectCopy.get("userId"));
														t._postModify();
													}}
												]);
											}
											t.require("password");
											t.passwordField.set("value","");
											t._postModify();
										}
										topic.publish("preventSave",false);
									},function() {
										actionShowMessage("There was an error verifying the new User ID. Please rechart the last value you entered.");
										topic.publish("preventSave",false);
										t.userIdField.set("value",t.objectCopy.get("userId"));
										t.resetPasswordField();
									});
								}
							}
						} else {
							t.resetPasswordField();
						}
					}
					
					var userId = t.object.get("userId");
					var disabled = (userId === "" ? true : false);
					t.disable(t.passwordField,disabled);
					t.disable("forcePasswordChange",disabled);
					t.disable("accessType",disabled);
					t.disable("mobileDeviceUser",disabled);
					t.disable("restrictAccess",disabled);
					//t.disable("types",disabled);
					//t.disable("primaryUserType",disabled);
					//t.disable("signatureUserType",disabled);
					//t.require("types",!disabled);
					
					if (dialNDocOn) {
						t.disable("dndAccessCode",disabled);
						t.disable("dndPasswordResetFlag",disabled);
					}
					
					viewController.refresh();
				});
				
				t.registerChangeLogic("accountStatus",function(value) {
					var t = this;
					var accountStatusPropertyModel = t.getPropertyModel("accountStatus");
					var displayValue = accountStatusPropertyModel.displayIndex[value];
					var textColor = (value == 1 ? "#950D1F" : "");
					t.accountStatusTextNode.style.color = textColor;
					t.accountStatusTextNode.innerText = displayValue;
				});
				
				t.registerChangeLogic("inactiveDate",function(value) {
					var t = this;
					var datePropertyModel = t.getPropertyModel("inactiveDate");
					var userPropertyModel = t.getPropertyModel("inactiveUser");
					t.statusInfoNode.innerHTML = (value === null ? "" : datePropertyModel.formatter(t.object.inactiveDate)+" by "+userPropertyModel.formatter(t.object.inactiveUser));
					
					arrayUtils.forEach(t.fields,function(field) {
						if (field.name !== "status") {
							field.set("disabled",(value !== null || t.actionType === "View" ? true : t._disabledFieldIndex[field.id] || false));
						}
					});
					
					var newEmployeeId = t.object.get("newEmployeeId");
					if (!t.inObjectLoad && value === null && newEmployeeId !== "") {
						var newEmployee = employeeStore.get(newEmployeeId);
						if (newEmployee && newEmployee.get("inactiveDate") === null) {
							actionShowMessage("Warning: this employee was changed to "+newEmployee.get("name")+" with employee ID "+newEmployee.get("id")+" who is currently active. Both accounts should not be active simultaneously except as a temporary measure in special circumstances.");
						}
					}
					
					if (value !== null) {
						t.object.set("accessType",2);
						t.availabilityEditor.set("actionType","View");
					} else if (t.availabilityEditor.object && t.actionType !== "View") {
						t.availabilityEditor.set("actionType","Edit");
					}
				});
				
				t.registerChangeLogic("contract",function(value) {
					var t = this;
					t.require("contractCompany",(value == 1 && TableManager.get("Tables.OutsideTables.LicensedVendor").data.length > 0 ? true : false));
				});
				
				t.registerChangeLogic("initials",function(value) {
					var t = this;
					if (!t.inObjectLoad) {
						DatabaseObjectModel.query("NDoc.User.EmployeeMapping",{initials:value,id:"!"+t.object.get("id")},["id"]).then(function(results) {
							if (results.length > 0) {
								actionShowMessage("Warning: already on file");
							}
						});
					}
				});
				
				var checkName = function() {
					var t = this;
					var firstName = t.object.get("firstName");
					var lastName = t.object.get("lastName");
					if (!t.inObjectLoad && firstName !== "" && lastName !== "") {
						DatabaseObjectModel.query("NDoc.User.EmployeeMapping",{
							firstName:firstName,
							lastName:lastName,
							middleInitial:t.object.get("middleInitial"),
							inactiveDate:"",
							id:"!"+t.object.get("id")
						},["id"]).then(function(results) {
							if (results.length > 0) {
								actionShowMessage("Warning: already on file");
							}
						});
					}
				};
				t.registerChangeLogic("firstName",checkName);
				t.registerChangeLogic("lastName",checkName);
				t.registerChangeLogic("middleInitial",checkName);
				
				t.registerChangeLogic("name",function() {
					var t = this;
					if (!t.inObjectLoad && t.object.get("name") !== t.objectCopy.get("name")) {
						DatabaseObjectModel.query("NDoc.User.EmployeeMapping",{
							name:t.object.get("name"),
							inactiveDate:"",
							id:"!"+t.object.get("id")
						},["id"]).then(function(results) {
							if (results.length > 0) {
								actionShowMessage("Warning: already on file");
							}
						});
						
						if (t.objectCopy.get("name") !== "") {
							actionShowMessage("Changes to the employee's name are reflected throughout the application, including old charting and reports. Do you wish to proceed?", [
								{label:"OK"},
								{label:"Cancel",callback:function() {
									t.object._changeAttrValue("name",t.objectCopy.get("name"));
								}}
							]);
						}
					}
				});
				
				t.registerChangeLogic("types",function(value) {
					var t = this;
					
					// System Administrator
					if (!t.inObjectLoad && arrayUtils.indexOf(value,993) !== -1 && arrayUtils.indexOf(currentEmployee.get("types"),993) === -1) {
						actionShowMessage("Only other "+sysAdminTypeName+" users can assign this user type.");
						value.splice(arrayUtils.indexOf(value,993),1);
						t.object._changeAttrValue("types",value);
					}
					
					// Security Manager
					if (!t.inObjectLoad && arrayUtils.indexOf(value,996) !== -1) {
						var allow = true;
						if (value.length > 1) {
							actionShowMessage("The security manager user type must be the only user type assigned");
							allow = false;
						} else if (t.object.get("teams").length > 0) {
							actionShowMessage("The security manager user type does not allow any teams to be assigned");
							allow = false;
						} else if (!t.object.get("restrictAccess")) {
							actionShowMessage("The security manager user type requires patient access to be restricted");
							allow = false;
						}
						if (allow) {
							
						} else {
							value.splice(arrayUtils.indexOf(value,996),1);
							t.object._changeAttrValue("types",value);
						}
					}
					
					if (arrayUtils.indexOf(value,996) !== -1) {
						t.disable("teams");
						t.disable("restrictAccess");
					} else if (t.object.get("userId") !== "") {
						t.disable("teams",false);
						t.disable("restrictAccess",false);
					}
					
					if (arrayUtils.indexOf(value,t.object.primaryUserType) === -1 && !t.inObjectLoad) {
						t.object.set("primaryUserType","");
					}
					if (arrayUtils.indexOf(value,t.object.signatureUserType) === -1 && !t.inObjectLoad) {
						t.object.set("signatureUserType","");
					}
					var propertyModel = t.getPropertyModel("primaryUserType");
					t.populateSelect(t.primaryUserTypeField,propertyModel.getOptionStore(),t.object.primaryUserType,false,value);
					t.require("primaryUserType",(value.length > 0));
					
					propertyModel = t.getPropertyModel("signatureUserType");
					t.populateSelect(t.signatureUserTypeField,propertyModel.getOptionStore(),t.object.signatureUserType,true,value);
					
					t.object.set("gpsUser",(arrayUtils.indexOf(value,997) !== -1));
				});
				
				t.registerChangeLogic("teams",function(value) {
					var t = this;
					if (arrayUtils.indexOf(value,t.object.primaryTeam) === -1 && !t.inObjectLoad) {
						t.object.set("primaryTeam","");
					}
					var propertyModel = t.getPropertyModel("primaryTeam");
					t.populateSelect(t.primaryTeamField,propertyModel.getOptionStore(),t.object.primaryTeam,false,value);
					t.require("primaryTeam",(value.length > 0));
					
					if (isHospice) {
						var teamPropertyModel = t.getPropertyModel("teams").itemModel;
						if (arrayUtils.some(value,function(teamId) {
							var team = teamPropertyModel.getTableItem(teamId);
							return (team && team.disciplineId == 9);
						})) {
							t.disable(t.contactSearch,false);
						} else {
							t.disable(t.contactSearch,true);
							var contacts = t.object.get("contacts");
							if (contacts && contacts.length > 0) {
								actionShowMessage("This employee has no volunteer teams selected. If the employee is saved without any volunteer teams, then all contact links will be cleared.");
								t.object.contacts = [];
							}
						}
					}
				});
				
				t.registerChangeLogic("zip",function(value) {
					var t = this;
					if (!t.inObjectLoad && value !== "") {
						topic.publish("preventSave",true);
						serverRequest("UI.Utility","GetCityAndState",{zipCode:value}).then(function(result) {
							topic.publish("preventSave",false);
							if (result.city !== "") {
								t.object.set("city",result.city);
							}
							if (result.state !== "") {
								t.object.set("state",result.state);
							}
						},function() {
							topic.publish("preventSave",false);
						});
					}
				});

				t.registerChangeLogic("fluVaccinationStatus",function(value) {
					t.require("fluVaccineDeclinedReason",false);
					t.require("fluVaccineReceivedAgency",false);
					t.require("fluVaccineReceivedOtherDoc",false);

					t.disable("fluVaccineReceivedAgency");
					t.disable("fluVaccineAdmAgencyDate");
					t.disable("fluVaccineReceivedOtherDoc");
					t.disable("fluVaccineAdmOtherDate");
					t.disable("fluVaccineAdmBy");
					t.disable("fluVaccineDeclined");
					t.disable("fluVaccineDeclinedReason");
					t.disable("fluVaccineDeclinedReasonOther");

					if (!t.inObjectLoad) {
						t.object.set("fluVaccineReceivedAgency","");
						t.fluVaccineReceivedAgencyField._updateSelection();
						t.object.set("fluVaccineReceivedOtherDoc","");
						t.fluVaccineReceivedOtherDocField._updateSelection();
						t.object.set("fluVaccineDeclinedReason","");
						t.object.set("fluVaccineDeclinedReasonOther","");
					}

					if (value == 1) { // Received
						t.require("fluStatusDate");
						t.require("fluVaccineReceivedAgency");
						t.require("fluVaccineReceivedOtherDoc");
						t.disable("fluVaccineReceivedAgency",false);
						t.disable("fluVaccineReceivedOtherDoc",false);
						if (!t.inObjectLoad) {
							t.object.set("fluVaccineDeclined",2);
						}
					} else if (value == 3) { // Declined
						t.require("fluStatusDate");
						t.require("fluVaccineDeclinedReason");
						t.disable("fluVaccineDeclinedReason",false);
						t.disable("fluVaccineDeclinedReasonOther",false);
						if (!t.inObjectLoad) {
							t.object.set("fluVaccineDeclined",1);
							t.object.set("fluVaccineReceivedAgency",2);
							t.object.set("fluVaccineReceivedOtherDoc",2);
						}
					} else if (value == 4) { // Unknown
						t.require("fluStatusDate",false);
						if (!t.inObjectLoad) {
							t.object.set("fluVaccineDeclined",2);
							t.object.set("fluVaccineReceivedAgency",2);
							t.object.set("fluVaccineReceivedOtherDoc",2);
						}
					}
				});

				t.registerChangeLogic("fluVaccineReceivedAgency",function(value) {
					t.disable("fluVaccineAdmAgencyDate",(value != 1));
					t.require("fluVaccineAdmAgencyDate",(value == 1));
					if (!t.inObjectLoad && value != 1) {
						t.object.set("fluVaccineAdmAgencyDate","");
						if (t.object.get("fluVaccinationStatus") == 1 && value == 2) {
							t.object.set("fluVaccineReceivedOtherDoc",1);
						}
					}
				});

				t.registerChangeLogic("fluVaccineReceivedOtherDoc",function(value) {
					t.disable("fluVaccineAdmOtherDate",(value != 1));
					t.disable("fluVaccineAdmBy",(value != 1));
					t.require("fluVaccineAdmOtherDate",(value == 1));
					t.require("fluVaccineAdmBy",(value == 1));
					if (!t.inObjectLoad && value != 1) {
						t.object.set("fluVaccineAdmOtherDate","");
						t.object.set("fluVaccineAdmBy","");
						if (t.object.get("fluVaccinationStatus") == 1 && value == 2) {
							t.object.set("fluVaccineReceivedAgency",1);
						}
					}
				});

				t.registerChangeLogic("fluVaccineDeclinedReason",function(value) {
					t.require("fluVaccineDeclinedReasonOther",(value == 7));
				});
				
				t.registerChangeLogic("startDate",function(value) {this.terminationDateField.set("constraints",{min:value}).validate();});
				t.registerChangeLogic("lic1FromDate",function(value) {this.lic1ToDateField.set("constraints",{min:value}).validate();});
				t.registerChangeLogic("lic2FromDate",function(value) {this.lic2ToDateField.set("constraints",{min:value}).validate();});
				t.registerChangeLogic("lic3FromDate",function(value) {this.lic3ToDateField.set("constraints",{min:value}).validate();});
				t.registerChangeLogic("lic4FromDate",function(value) {this.lic4ToDateField.set("constraints",{min:value}).validate();});
				t.registerChangeLogic("lic5FromDate",function(value) {this.lic5ToDateField.set("constraints",{min:value}).validate();});
				t.registerChangeLogic("lic6FromDate",function(value) {this.lic6ToDateField.set("constraints",{min:value}).validate();});
				t.registerChangeLogic("lic7FromDate",function(value) {this.lic7ToDateField.set("constraints",{min:value}).validate();});
				
				if (isHospice) {
					var results = t.contactStore.query({});
					results.observe(function(object,removedFrom,insertedInto) {
						if (removedFrom > -1) {
							t.removeField("contact"+object.id+"RemoveButton");
							domConstruct.destroy("contact-"+object.id);
							registry.byId("contact"+object.id+"RemoveButton").destroy();
						}
						if (insertedInto > -1) {
							var button = new Button({
								id: "contact"+object.id+"RemoveButton",
								label: "-",
								onClick: function() {
									t.contactStore.remove(object.id);
								},
								title: "Click to remove this contact",
								disabled: (t.actionType === "View")
							});
							var field = new TextBox({
								value: object.name,
								readOnly: true
							});
							var node = domConstruct.toDom("<div id='contact-"+object.id+"'></div>");
							domConstruct.place(field.domNode,node);
							domConstruct.place(button.domNode,node);
							domConstruct.place(node,t.contactContainerNode,"first");
							t.fields.push(button);
						}
					});
				}
				
				if (payrollOn) {
					t.registerChangeLogic("costCenters",function(value) {
						var t = this;
						if (arrayUtils.indexOf(value,t.object.primaryCostCenter) === -1 && !t.inObjectLoad) {
							t.object.set("primaryCostCenter","");
						}
						var propertyModel = t.getPropertyModel("primaryCostCenter");
						t.populateSelect(t.primaryCostCenterField,propertyModel.getOptionStore(),t.object.primaryCostCenter,false,value);
						t.require("primaryCostCenter",(value.length > 0));
					});
				}

				if (geocodeFieldsOn) {
					/*t.registerChangeLogic("addressCoordinates", function(value) {
						debugger;
						t.object.set("street2", value);
					});*/

					var gpsLookupButton = new Button({
						label: "Look Up GPS",
						id: "gpsLookupButton",
						disabled: (t.actionType === "View"),
						onClick: function() {
							var lookupUrl = "https://www.melissa.com/v2/lookups/geocoder/address?address=";
							var addressString = t.object.get("street1") + ", " + t.object.get("city") + ", " + t.object.get("state") + " " + t.object.get("zip");
							// Replace spaces with plus characters. Keep them unencoded, but encode commas
							addressString = encodeURI(addressString.replace(/\s/g,"+")).replace(/,/g,"%2C")
							window.open(lookupUrl + addressString, "lookupWindow");
						}
					});

					var showMapButton = new Button({
						label: "Show Map",
						id: "showMapButton",
						disabled: (t.actionType === "View"),
						onClick: function() {
							var mapUrl = "https://www.google.com/maps/place/";
							var coordinates = encodeURI(t.object.get("addressCoordinates"));
							window.open(mapUrl + coordinates, "mapWindow");
						}
					});

					var getGpsButton = new Button({
						label: "Get GPS",
						id: "getGpsButton",
						disabled: (t.actionType === "View"),
						onClick: function() {
							if (!t.addressCoordinatesField.get("disabled")) {
								var options = {
									street: t.object.get("street1"),
									city: t.object.get("city"),
									state: t.object.get("state"),
									zip: t.object.get("zip")
								};

								serverRequest("Patient.Visit.Utility", "CalculateAddressCoordinates", options, 1).then(function(result) {
									if (!result.coordinates) {
										result.coordinates = "";
									}
									
									if (result.coordinates != t.object.get("addressCoordinates")) {
										t.object.set("addressCoordinates", result.coordinates);
										t.set("modified",true);
									}

									if (result.error) {
										actionShowMessage(result.error);
									}
								}, function(error) {
									actionShowMessage(error);
								});
							}
						}
					});

					domConstruct.place(getGpsButton.domNode, t.geocodeFieldsDiv, "last");
					domConstruct.place(showMapButton.domNode, t.geocodeFieldsDiv, "last");
					domConstruct.place(gpsLookupButton.domNode, t.geocodeFieldsDiv, "last");
				}
			},
			
			_getObjectAttr: function() {
				var t = this;
				var object = t.inherited(arguments);
				
				if (isHospice) {
					object.contacts = [];
					arrayUtils.forEach(t.contactStore.data,function(contact) {
						object.contacts.push(contact);
					});
				}
				
				var password = t.passwordField.get("value");
				if (password !== "********" && password !== "") {
					object.password = password;
				}
				
				return object;
			},
			
			_idOnValidate: function(value) {
				var t = this;
				var deferred = new Deferred();
				if (value !== "" && !(/^[A-Z0-9]+$/i).test(value)) {
					actionShowMessage("Enter alphanumeric characters only")
					deferred.resolve(false);
				} else if (value !== t.objectCopy.get("id")) {
					topic.publish("preventSave",true);
					DatabaseObjectModel.query("NDoc.User.EmployeeMapping",{id:value},["name","inactiveDate"]).then(function(results) {
						var valid = true;
						var message = "";
						if (results.length > 0) {
							if (results[0].inactiveDate === "") {
								actionShowMessage("Invalid: duplicate employee ID. Assigned to "+results[0].name);
								valid = false;
							} else {
								message = "This ID was assigned to inactive employee "+results[0].name;
							}
						}
						if (valid) {
							if (t.objectCopy.get("id") !== "") {
								message += (message === "" ? "" : ".<br><br>") + "Warning: you are changing this employee's ID. This ID is what identifies the employee in billing. Existing records will remain with the employee ID they were charted under. Do you wish to proceed?";
								
								actionShowMessage(message,[
									{label:"OK"},
									{label:"Cancel",callback:function() {
										t.object.set("id",t.objectCopy.get("id"));
										t._postModify();
									}}
								]);
							} else if (message !== "") {
								actionShowMessage(message);
							}
						}
						deferred.resolve(valid);
						topic.publish("preventSave",false);
					},function() {
						actionShowMessage("There was an error verifying the new Employee ID. Please rechart the last value you entered.");
						topic.publish("preventSave",false);
						deferred.resolve(false);
					});
				} else {
					deferred.resolve(true);
				}
				return deferred.promise;
			},
			
			_userIdOnValidate: function(value) {
				var t = this;
				var valid = true;
				if (value !== "" && !(/^[a-zA-Z0-9]+$/i).test(value)) {
					actionShowMessage("Enter alphanumeric characters only")
					valid = false;
				}
				return valid;
			},
			
			_emailAddressOnValidate: function(value) {
				topic.publish("preventSave",true);
				return serverRequest("UI.Utility","ValidateEmailAddress",{emailAddress:value}).then(function(valid) {
					if (!valid) {
						actionShowMessage("Please enter a valid email address");
					}
					topic.publish("preventSave",false);
					return valid;
				},function() {
					actionShowMessage("There was an error verifying the Email Address. Please rechart the last value you entered.");
					topic.publish("preventSave",false);
					return false;
				});
			},
			
			_npiOnValidate: function(value) {
				topic.publish("preventSave",true);
				return serverRequest("UI.Utility","ValidateNPI",value).then(function(valid) {
					if (!valid) {
						actionShowMessage("Please enter a valid NPI");
					}
					topic.publish("preventSave",false);
					return valid;
				},function() {
					actionShowMessage("There was an error verifying the NPI. Please rechart the last value you entered.");
					topic.publish("preventSave",false);
					return false;
				});
			},
			
			_teamsOnValidate: function(value) {
				var valid = true;
				if (value.length > 7) {
					valid = false;
					actionShowMessage("Up to 7 teams are allowed");
				}
				return valid;
			},
			
			_typesOnValidate: function(value) {
				var valid = true;
				if (value.length > 6) {
					valid = false;
					actionShowMessage("Up to 6 types are allowed");
				}
				return valid;
			},
			
			_dndAccessCodeOnValidate: function(value) {
				var t = this;
				var deferred = new Deferred();
				topic.publish("preventSave",true);
				DatabaseObjectModel.query("NDoc.User.EmployeeMapping",{id:"!"+t.object.get("id"),dndAccessCode:value},["name"]).then(function(results) {
					if (results.length > 0) {
						actionShowMessage("The Dial-N-Document access code is already assigned to "+results[0].name);
					}
					deferred.resolve((results.length === 0));
					topic.publish("preventSave",false);
				},function() {
					actionShowMessage("There was an error verifying the new Dial-N-Document Access Code. Please rechart the last value you entered.");
					topic.publish("preventSave",false);
					deferred.resolve(false);
				});
				return deferred.promise;
			},
			
			resetPasswordField: function() {
				var t = this;
				t.passwordField._onChangeActive = false;
				t.passwordField.set("value",(t.objectCopy.get("userId") !== "" ? "********" : ""));
				t.passwordField._onChangeActive = true;
			},
			
			createAvailabilityEditor: function() {
				var t = this;

				t.own(on(t.availabilityCalendar,"change",lang.hitch(t,"updateAvailability")));
				
				t.availabilityModel = new DatabaseObjectModel({
					databaseClass: "NDoc.Availability.EmployeeAvailability"
				});
				
				t.availabilityModel.loadPromise.then(function() {
					t.Availability = t.availabilityModel.getObject();
					
					var updateAvailabilityActions = function() {
						domStyle.set(t.availabilityAddButton.domNode,"display","none");
						domStyle.set(t.availabilityEditButton.domNode,"display","none");
						domStyle.set(t.availabilityDeleteButton.domNode,"display","none");
						domStyle.set(t.availabilityCancelButton.domNode,"display","none");
						domStyle.set(t.availabilitySaveButton.domNode,"display","none");
						
						if (t.availabilityStack.selectedChildWidget === t.availabilityGridPane) {
							domStyle.set(t.availabilityAddButton.domNode,"display","");
							var selectedIds = t.availabilityGrid.select.row.getSelected();
							if (selectedIds.length === 1) {
								domStyle.set(t.availabilityEditButton.domNode,"display","");
								domStyle.set(t.availabilityDeleteButton.domNode,"display","");
							}
						} else {
							domStyle.set(t.availabilityCancelButton.domNode,"display","");
							domStyle.set(t.availabilitySaveButton.domNode,"display","");
						}
					};
					
					t.fields.push(t.availabilityAddButton);
					t.fields.push(t.availabilityEditButton);
					t.fields.push(t.availabilityDeleteButton);
					
					t.own(on(t.availabilityAddButton,"click",function() {
						if (t.object.get("id") !== "" && t.object.get("id") === t.objectCopy.get("id")) {
							t.availabilityGrid.select.row.clear();
							var availability = new t.Availability({
								Entity: t.object.id+",User.Employee",
								Available: true,
								AllDay: false,
								RepeatOnce: false,
								RepeatEveryOther: false
							});
							t.availabilityEditor.set("object",availability);
							t.availabilityEditor.set("actionType","Edit");
							t.availabilityStack.selectChild(t.availabilityEditorPane);
							updateAvailabilityActions();
						} else {
							actionShowMessage("Please save the employee before changing availability (due to this being a new employee or an employee ID change)");
						}
					}));
					
					t.own(on(t.availabilityEditButton,"click",function() {
						if (t.object.get("id") !== "" && t.object.get("id") === t.objectCopy.get("id")) {
							var selectedIds = t.availabilityGrid.select.row.getSelected();
							var object = t.availabilityGrid.store.get(selectedIds[0]);
							t.availabilityEditor.set("object",lang.clone(object));
							t.availabilityEditor.set("actionType","Edit");
							t.availabilityStack.selectChild(t.availabilityEditorPane);
							updateAvailabilityActions();
						} else {
							actionShowMessage("Please save the employee before changing availability (due to this being a new employee or an employee ID change)");
						}
					}));
					
					t.own(on(t.availabilitySaveButton,"click",function() {
						if (t.availabilityEditor.okToSave()) {
							t.availabilitySaveButton.set("disabled",true);
							t.availabilityCancelButton.set("disabled",true);
							t.availabilityEditor.object.save().then(function() {
								t.availabilitySaveButton.set("disabled",false);
								t.availabilityCancelButton.set("disabled",false);

								t.availabilityGrid.store.put(t.availabilityEditor.object);
								t.availabilityGrid.model.clearCache();
								t.availabilityGrid.body.refresh();
								
								var found = false;
								var availability = t.object.availability;
								for (var i = 0; i < availability.length && !found; i++) {
									if (availability[i].id === t.availabilityEditor.object.id) {
										availability[i] = t.availabilityEditor.object;
										t.objectCopy.availability[i] = t.availabilityEditor.object.getData();
										found = true;
									}
								}
								if (!found) {
									var data = t.availabilityEditor.object.getData();
									t.object.availability.push(data);
									t.objectCopy.availability.push(data);
								}
								
								t.availabilityGrid.select.row.clear();
								t.updateAvailability();
								t.availabilityStack.selectChild(t.availabilityGridPane);
								updateAvailabilityActions();
								
								topic.publish("status",{message:"Availability saved",duration:1500,type:"message"});
							},function(error) {
								console.log(error);
								topic.publish("status",{message:"There was a problem saving the availability",duration:1500,type:"error"});
								t.availabilitySaveButton.set("disabled",false);
								t.availabilityCancelButton.set("disabled",false);
							});
						}
					}));
					
					t.own(on(t.availabilityDeleteButton,"click",function() {
						if (t.object.get("id") !== "" && t.object.get("id") === t.objectCopy.get("id")) {
							actionShowMessage("Are you sure you want to delete this availability item?",[{
								label: "Yes",callback: function() {
									t.availabilityEditButton.set("disabled",true);
									t.availabilityDeleteButton.set("disabled",true);

									var selectedIds = t.availabilityGrid.select.row.getSelected();
									var object = t.availabilityGrid.store.get(selectedIds[0]);
									object.remove().then(function() {
										t.availabilityEditButton.set("disabled",false);
										t.availabilityDeleteButton.set("disabled",false);

										t.availabilityGrid.store.remove(object.id);
										t.availabilityGrid.model.clearCache();
										t.availabilityGrid.body.refresh();
										
										// remove from employee editor's object's availability array
										var availability = t.object.availability;
										for (var i = 0; i < availability.length; i++) {
											if (availability[i].id === object.id) {
												availability.splice(i,1);
												t.objectCopy.availability.splice(i,1);
												break;
											}
										}
										
										t.availabilityGrid.select.row.clear();
										t.updateAvailability();
									},function(error) {
										actionShowMessage(error);
										t.availabilityEditButton.set("disabled",false);
										t.availabilityDeleteButton.set("disabled",false);
									});
							}},{label: "No",callback: function() {}}]);
						} else {
							actionShowMessage("Please save the employee before changing availability (due to this being a new employee or an employee ID change)");
						}
					}));
					
					t.own(on(t.availabilityCancelButton,"click",function() {
						if (t.availabilityEditor.get("modified")) {
							actionShowMessage("The most recent open availability change was not saved.");
						}
						
						t.availabilityStack.selectChild(t.availabilityGridPane);
						updateAvailabilityActions();
					}));
					
					t.availabilityGrid = new Grid({
						cacheClass: Cache,
						store: new Memory(),
						structure: t.availabilityModel.getGridLayout(["Available","AllDay","StartDate","EndDate","RepeatOnce","RepeatEveryOther","RepeatDays","StartTime","EndTime","Note"]),
						style:"font-size:0.8em",
						autoHeight: true,
						modules: [
							SingleSort,
							SelectRow,
							IndirectSelectColumn,
							ExtendedSelectRow,
							Filter
						],
						modelExtensions: [
							FormatSort,
							"gridx/core/model/extensions/Sort",
							"gridx/core/model/extensions/Query"
						]
					});
					t.availabilityGridPane.domNode.appendChild(t.availabilityGrid.domNode);
					t.availabilityGrid.startup();
					
					t.availabilityGrid.connect(t.availabilityGrid,"onRowClick",gridRowClickHandler);
					
					t.availabilityGrid.connect(t.availabilityGrid.select.row,"onSelectionChange",function(newSelectedIds) {
						updateAvailabilityActions();
					});
					
					t.availabilityEditor = new ObjectEditor({
						actionType:"View",
						objectModel: t.availabilityModel,
						parentEditor: t,
						templateString: 
							'<div class="content-area" >\
								<div data-dojo-attach-point="formNode" data-dojo-type="dijit/form/Form">\
									<div class="content-flex-container">\
										<div class="flex-field-container">\
											<div class="field-name field-name-medium">\
												<label>Available?</label>\
											</div>\
											<div class="field-value field-value-large">\
												<input data-dojo-attach-point="AvailableNode">\
											</div>\
										</div>\
										<div class="flex-field-container">\
											<div class="field-name field-name-medium">\
												<label>All Day?</label>\
											</div>\
											<div class="field-value field-value-large">\
												<input data-dojo-attach-point="AllDayNode">\
											</div>\
										</div>\
									</div>\
									<div class="content-flex-container">\
										<div class="flex-field-container">\
											<div class="field-name field-name-medium">\
												<label>Start Date</label>\
											</div>\
											<div class="field-value field-value-large">\
												<input data-dojo-attach-point="StartDateNode">\
											</div>\
										</div>\
									</div>\
									<div class="content-flex-container">\
										<div class="flex-field-container">\
											<div class="field-name field-name-medium">\
												<label>End Date</label>\
											</div>\
											<div class="field-value field-value-large">\
												<input data-dojo-attach-point="EndDateNode">\
											</div>\
										</div>\
									</div>\
									<div class="content-flex-container">\
										<div class="flex-field-container">\
											<div class="field-name field-name-medium">\
												<label>Repeat</label>\
											</div>\
											<div class="field-value field-value-large">\
												<input data-dojo-attach-point="RepeatOnceNode"><label>Once</label>\
											</div>\
										</div>\
									</div>\
									<div class="content-flex-container">\
										<div class="flex-field-container">\
											<div class="field-name field-name-medium">\</div>\
											<div class="field-value field-value-large">\
												<input data-dojo-attach-point="RepeatEveryOtherNode"><label>Every Other Week</label>\
											</div>\
										</div>\
									</div>\
									<div class="content-flex-container">\
										<div class="flex-field-container">\
											<div class="field-name field-name-medium">\</div>\
											<div class="field-value field-value-large">\
												<input data-dojo-attach-point="RepeatDaysNode">\
											</div>\
										</div>\
									</div>\
									<div class="content-flex-container">\
										<div class="flex-field-container">\
											<div class="field-name field-name-medium">\
												<label>Start Time</label>\
											</div>\
											<div class="field-value field-value-large">\
												<input data-dojo-attach-point="StartTimeNode">\
											</div>\
										</div>\
									</div>\
									<div class="content-flex-container">\
										<div class="flex-field-container">\
											<div class="field-name field-name-medium">\
												<label>End Time</label>\
											</div>\
											<div class="field-value field-value-large">\
												<input data-dojo-attach-point="EndTimeNode">\
											</div>\
										</div>\
									</div>\
									<div class="content-flex-container">\
										<div class="flex-field-container">\
											<div class="field-name field-name-medium">\
												<label>Note</label>\
											</div>\
											<div class="field-value field-value-grand">\
												<input data-dojo-attach-point="NoteNode">\
											</div>\
										</div>\
									</div>\
								</div>\
							</div>',
							
						postMixInProperties: function() {
							var t = this;
							t.inherited("postMixInProperties",arguments);
							
							t.getPropertyModel("RepeatDays").fieldProperties["multiple"] = true;
							t.getPropertyModel("RepeatDays").getField = function(srcNodeRef) {
								return new CheckedMultiSelect(this.getFieldProperties(),srcNodeRef);
							};
							
							t.getPropertyModel("Note").getField = function(node) {
								return new Textarea(this.getFieldProperties(),node);
							};
						},
						
						postCreate: function() {
							var t = this;
							t.inherited("postCreate",arguments);
							
							t.own(aspect.after(t,"_setActionTypeAttr",function(value) {
								t.parentEditor.idField.set("disabled",(t.get("actionType") !== "View" || t.parentEditor.get("actionType") === "View" || t.parentEditor.object.get("inactiveDate") !== null));
							}));
							
							t.registerChangeLogic("StartDate",function(value) {this.EndDateField.set("constraints",{min:value}).validate();});
							t.registerChangeLogic("StartTime",function(value) {
								var constraints = this.EndTimeField.constraints;
								constraints.min = value;
								this.EndTimeField.set("constraints",constraints).validate();
							});

							this.EndTimeField.isInRange = function() {
								return locale.format(this.get('value'),{selector:"time",timePattern:"HHmm"}) === "0000" || this.rangeCheck(this.get('value'), this.constraints);
							};
							
							t.registerChangeLogic("AllDay",function(value) {
								value = value || false;
								if (value || !t.inObjectLoad) {
									t.require("StartDate",value);
									t.require("EndDate",value);
									t.disable("StartTime",value);
									t.disable("EndTime",value);
									t.disable("RepeatOnce",value);
									t.disable("RepeatEveryOther",value);
									t.disable("RepeatDays",value);
									t.object.set("StartTime",null);
									t.object.set("EndTime",null);
								}
								t.require("StartTime",!value);
								t.require("EndTime",!value);
							});
							
							t.registerChangeLogic("RepeatOnce",function(value) {
								value = value || false;
								if (value || !t.inObjectLoad) {
									t.require("StartDate",value);
									t.disable("EndDate",value);
									t.disable("AllDay",value);
									t.disable("RepeatEveryOther",value);
									t.disable("RepeatDays",value);
									if (value) {
										t.object.set("EndDate",t.StartDateField.get("value"));
									}
								}
							});
							
							t.registerChangeLogic("RepeatEveryOther",function(value) {
								value = value || false;
								if (value || !t.inObjectLoad) {
									t.require("StartDate",value);
									t.disable("AllDay",value);
									t.disable("RepeatOnce",value);
								}
							});
							
							t.registerChangeLogic("RepeatDays",function(value) {
								value = (value.length > 0);
								if (value || !t.inObjectLoad) {
									t.disable("AllDay",value);
									t.disable("RepeatOnce",value);
								}
							});
							
							t.registerChangeLogic("StartDate",function(value) {
								if (t.object.RepeatOnce) {
									t.object.set("EndDate",value);
								}
							});
						}
					});
					t.availabilityEditorPane.domNode.appendChild(t.availabilityEditor.domNode);
				});
			},
			
			updateAvailability: function() {
				var t = this;
				var selectedDate = t.availabilityCalendar.get("value");
				serverRequest("NDoc.User.Employee","GetAvailabilityExternal",{
					entity: t.object.id+",User.Employee",
					startDate: locale.format(selectedDate,{selector:"date",datePattern:"yyyy-MM-dd"})
				}).then(function(results) {
					domConstruct.empty(t.availabilityContainer);
					var node = domConstruct.create("div",{innerHTML:locale.format(selectedDate,{selector:"date",datePattern:"MMMM d, y"})},t.availabilityContainer);
					domClass.add(node,"content-area-title-text");
					
					node = domConstruct.create("div",{innerHTML:"Available:"},t.availabilityContainer);
					domStyle.set(node,"margin-left","15px");
					domStyle.set(node,"margin-top","5px");
					domClass.add(node,"field-name");
					
					arrayUtils.forEach(results,function(result) {
						var startTime = locale.format(locale.parse(result.start,{selector:"date",datePattern:"yyyy-MM-dd HH:mm:ss"}),{selector:"time",timePattern:"HHmm"});
						var endTime = locale.format(locale.parse(result.end,{selector:"date",datePattern:"yyyy-MM-dd HH:mm:ss"}),{selector:"time",timePattern:"HHmm"});
						var subNode = domConstruct.create("div",{innerHTML:startTime+" - "+endTime},node);
						domStyle.set(subNode,"margin-left","25px");
					});
					
					t.availabilityBorderContainer.resize();
				},function() {
					actionShowMessage("There was an error updating employee availability. Please try again.");
				});
			},
			
			startup: function() {
				var t = this;
				t.inherited(arguments);
				t.resize();
			},
			
			resize: function() {
				var t = this;
				t.inherited(arguments);
				
				var containerBox = domGeom.getContentBox(t.containerNode);
				var header = domGeom.getContentBox(t.headerDiv);
				var height = containerBox.h - header.h - 12;
				if (height < 40) {
					height = 40;
				}
				domStyle.set(t.scrollableDiv,"height",height + "px");
				t.tabContainer.resize();
				t.teamsField.resize();
			},
			
			show: function(object) {
				var t = this;
				var demoPane = t.tabContainer.getChildren()[0];
				demoPane.domNode.scrollTop = 0;
				t.tabContainer.selectChild(demoPane);
			}
        });
		
		return EmployeeEditor;
	}
);